"viterbi" <-
function(x, p, a, du, pi, L, penalty="BIC", b.limit=0.001, cut=0.5){

    check.input(x, p, a, du, pi)
    
    ## number of observiations
    n = length(x)
    
    ## number of states
    m = nrow(a)
    
    ## maximum duration
    D  = nrow(du)
    
    ## make sure p start from a value < step
    ## sStart = the step before current data
    sStart = floor(min(p)/L) 
    pStart = sStart*L
    p = p - pStart
    
    ## how many steps altogether
    Z = ceiling(max(p)/L)
    
    ## step indicator, every 'step' bp is one step
    stp = ceiling(p/L)
    
    ## check that for the input data, every step have at least 3 observations
    len = tapply(p, stp, length)
    if(any(len<3)){
        stop("there is at least one step with less than 3 observations\n")
    }
    
    ## indicator of end of every step
    dif = diff(stp)
    stp.end = c(which(dif>0), length(stp)) - 1

    path = numeric(Z)
    segment = matrix(0, nrow=Z+1, ncol=3)
    dims  = c(n, m, D, Z, L)
    logL = 0.0

    if(penalty=="BIC"){
        penal = 2
    }else if(penalty=="AIC"){
        penal = 1
    }else{
        penal = 0
    }

    ## indicator of duration 
    di = (du>0)
    
    n.seg = 0 ## number of segment
    SW = .C("viterbi", as.double(x), as.double(p), as.integer(stp.end),
            as.integer(dims), path=as.double(path), segment=as.double(segment), 
            nSeg=as.integer(n.seg), logL=as.double(logL), as.double(t(a)), 
            as.double(t(du)),  as.integer(t(di)), as.double(pi), 
            as.integer(penal), as.double(b.limit), as.double(cut), 
            PACKAGE="ss.hmm");
    
    # -----------------------------------------
    # segment
    # 1st col of segment = end postion, 
    # 2nd col of segment = signal at end postion
    # 3rd col of segment = state
    # -----------------------------------------
    segment = matrix(SW$segment, byrow=TRUE, nrow=Z+1, ncol=3)
    n.seg   = SW$nSeg
    segment = data.frame(segment[(Z-n.seg+2):(Z+1),])
    names(segment) = c("p", "x", "state")
    segment$state  = segment$state + 1
    
    # -----------------------------------------
    # add back the baseline of position
    # -----------------------------------------
    p = p + pStart
    segment$p = segment$p + pStart

    path = SW$path
    logL = SW$logL
    ## states are indexed from 0 in c, so add 1 here
    path = path + 1
    bestPath = list(path=path, L=L, x=x, p=p, segment=segment, logL=logL)
    class(bestPath) = "hmm.path"
    
    bestPath
}
