"emiss" <-
function(xk, pk, x.fix, p.fix, state,penalty="BIC",b.limit=1/1000,log.zero=-100)
{    
    ## other wise
    n   = length(xk)

    ## if the previous state specification is impossible, return 0
    if(!is.null(x.fix) && is.na(x.fix)){
        return(rep(log.zero*n, 4))
    }
    
    em = numeric(4)
    adj = sqrt((n-1)/n)

    x.bar = mean(xk)
    p.bar = mean(pk)


    if(state[1] || state[2]){
        # ----------------------------------------------------------
        # state 1 or 2, flat line
        # ---------------------------------------------------------- 
        if(is.null(x.fix)){
            mean.x = x.bar
        }else{
            mean.x = x.fix
        }
        
        sd.x   = sqrt(sum((xk-mean.x)^2)/n)
        em[1] = em[2] = sum(dnorm(xk, mean.x, sd.x, log=TRUE))
    }
    if(state[3] || state[4]){
        # ----------------------------------------------------------
        # state 3 or 4, line with slope
        #
        # x = a + b*p             (1)
        # x.bar = a + b*p.bar     (2)
        # x.fix = a + b*p.fix     (3)
        # 
        # (1) + (2) => x-x.bar = b*(p - p.bar)
        # (1) + (3) => x-x.fix = b*(p - p.fix)
        # ----------------------------------------------------------
        
        ## get nx and np so that nx = b*np
        if(is.null(x.fix)){
            nx = xk - x.bar
            np = pk - p.bar
        }else{
            nx = xk - x.fix
            np = pk - p.fix
        }
        
        ## s is estimation of slope b
        s = sum(nx*np)/sum(np*np)
        b = s
        if(s >= 0 && s <= b.limit){ b = b.limit }
        if(s <  0 && s > -b.limit){ b = -b.limit }    
        
        if(penalty=="BIC"){
            penalt = - 0.5*log(n)
        }else if(penalty=="AIC"){
            penalt = - 1
        }else{
            stop("unknown penalty specification")
        }

        if(!state[4]){
            ## if only calculate emission probability of state 3
            if(s>=0){
                em[3] = log.zero*n
            }else{
                resid = nx - b*np
                sd.r = sqrt(sum(resid^2)/n)
                em[3] = sum(dnorm(resid, 0, sd.r, log=TRUE)) - penalt
            }
        } 
        if(!state[3]){
            ## if only calculate emission probability of state 4
            if(s<=0){
                em[4] = log.zero*n
            }else{
                resid = nx - b*np
                sd.r = sqrt(sum(resid^2)/n)
                em[4] = sum(dnorm(resid, 0, sd.r, log=TRUE)) - penalt
            }
        }
        if(state[3] && state[4]){
            ## calculate emission probability of both state 3 and 4
            resid = nx - b*np
            sd.r = sqrt(sum(resid^2)/n)
            if(s>=0){
                em[3] = log.zero*n
                em[4] = sum(dnorm(resid, 0, sd.r, log=TRUE)) - penalt
            }else{
                em[4] = log.zero*n
                em[3] = sum(dnorm(resid, 0, sd.r, log=TRUE)) - penalt
            }
        }
    }
    return(em)

}
