# -------------------------------------------------------------------------- #
# posterior probability
#
# fi[t1,t2] = P(O_1, ..., O_t, S_i at t1:t2 | lambda)
# bi[t1,t2] = P(O_{t+1}, ..., O_T | S_i at t1:t2, lambda)
#
# -------------------------------------------------------------------------- #

"post.prob" <-
function(f, b, D){

    f1 = f[[1]]
    f2 = f[[2]]
    f3 = f[[3]]
    f4 = f[[4]]

    b1 = b[[1]]
    b2 = b[[2]]
    b3 = b[[3]]
    b4 = b[[4]]

    Z = nrow(f1)
    
    post.p = matrix(0, nrow=Z, ncol=4)
    
    for(t in 1:Z){
        t1s = max(t-D+1, 1):t
        v1 = v2 = v3 = v4 = numeric(length(t1s))
        c = 0
        for(t1 in t1s){
            c = c + 1
            t2s = t:min(t1+D-1, Z)
            v1[c] = logsumexp(f1[t1,t2s] + b1[t1,t2s])
            v2[c] = logsumexp(f2[t1,t2s] + b2[t1,t2s])
            v3[c] = logsumexp(f3[t1,t2s] + b3[t1,t2s])
            v4[c] = logsumexp(f4[t1,t2s] + b4[t1,t2s])
        }
        post.p[t,1] = logsumexp(v1)
        post.p[t,2] = logsumexp(v2)
        post.p[t,3] = logsumexp(v3)
        post.p[t,4] = logsumexp(v4)
    }

    post.p
}

